// FileTypeDiagDlg.cpp : implementation file
//

#include "stdafx.h"
#include "FileTypeDiag.h"
#include "FileTypeDiagDlg.h"
#include "LeoHelpers.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#endif


// CFileTypeDiagDlg dialog

CFileTypeDiagDlg::CFileTypeDiagDlg(CWnd* pParent /*=NULL*/)
	: CDialog(CFileTypeDiagDlg::IDD, pParent)
	, m_cstrExtension(_T(""))
{
	m_hIcon = AfxGetApp()->LoadIcon(IDR_MAINFRAME);
}

void CFileTypeDiagDlg::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);
	DDX_Text(pDX, IDC_EDIT_EXTENSION, m_cstrExtension);
}

BEGIN_MESSAGE_MAP(CFileTypeDiagDlg, CDialog)
	ON_WM_PAINT()
	ON_WM_QUERYDRAGICON()
	//}}AFX_MSG_MAP
	ON_BN_CLICKED(IDOK, &CFileTypeDiagDlg::OnBnClickedOk)
END_MESSAGE_MAP()


// CFileTypeDiagDlg message handlers

BOOL CFileTypeDiagDlg::OnInitDialog()
{
	CDialog::OnInitDialog();

	// Set the icon for this dialog.  The framework does this automatically
	//  when the application's main window is not a dialog
	SetIcon(m_hIcon, TRUE);			// Set big icon
	SetIcon(m_hIcon, FALSE);		// Set small icon

	GetDlgItem(IDC_EDIT_EXTENSION)->SetFocus();

	return FALSE;  // return TRUE  unless you set the focus to a control
}

// If you add a minimize button to your dialog, you will need the code below
//  to draw the icon.  For MFC applications using the document/view model,
//  this is automatically done for you by the framework.

void CFileTypeDiagDlg::OnPaint()
{
	if (IsIconic())
	{
		CPaintDC dc(this); // device context for painting

		SendMessage(WM_ICONERASEBKGND, reinterpret_cast<WPARAM>(dc.GetSafeHdc()), 0);

		// Center icon in client rectangle
		int cxIcon = GetSystemMetrics(SM_CXICON);
		int cyIcon = GetSystemMetrics(SM_CYICON);
		CRect rect;
		GetClientRect(&rect);
		int x = (rect.Width() - cxIcon + 1) / 2;
		int y = (rect.Height() - cyIcon + 1) / 2;

		// Draw the icon
		dc.DrawIcon(x, y, m_hIcon);
	}
	else
	{
		CDialog::OnPaint();
	}
}

// The system calls this function to obtain the cursor to display while the user drags
//  the minimized window.
HCURSOR CFileTypeDiagDlg::OnQueryDragIcon()
{
	return static_cast<HCURSOR>(m_hIcon);
}

// From http://blogs.msdn.com/oldnewthing/archive/2005/02/01/364563.aspx
bool Is64BitWindows()
{
#if defined(_WIN64)
	return true;  // 64-bit programs run only on Win64
#elif defined(_WIN32)
	// 32-bit programs run on both 32-bit and 64-bit Windows so must sniff
	BOOL f64 = FALSE;
	return (::IsWow64Process(::GetCurrentProcess(), &f64) && f64);
#else
	return false; // Win64 does not support Win16
#endif
}


void FormatToStdio(CStdioFile &fileOut, const wchar_t *szFormat, ...)
{
	va_list args;
	va_start(args, szFormat);
	wchar_t *szBuf = LeoHelpers::VStringAllocAndFormat(szFormat, args);
	va_end(args);

	if (szBuf != NULL)
	{
		fileOut.WriteString(szBuf);
		delete[] szBuf;
	}
}

typedef std::set< std::wstring, LeoHelpers::LessUpperCase > SetNoCase;

const wchar_t *getRootName(HKEY hkRoot)
{
	if (hkRoot == HKEY_CURRENT_USER)
	{
		return L"HKCU";
	}

	if (hkRoot == HKEY_LOCAL_MACHINE)
	{
		return L"HKLM";
	}

	if (hkRoot == HKEY_CLASSES_ROOT)
	{
		return L"HKCR";
	}

	return L"<bad root>";
}

const wchar_t *getSamName(REGSAM sam)
{
	if (sam & KEY_WOW64_32KEY)
	{
		return L"32";
	}

	if (sam & KEY_WOW64_64KEY)
	{
		return L"64";
	}

	return L"00";
}

void writeExpLocStr(CStdioFile &fileOut, const wchar_t *szInStr)
{
	if (szInStr[0] == L'@')
	{
		std::wstring str = szInStr;
		if (LeoHelpers::ConvertRegistryStyleLocalisedString(str))
		{
			fileOut.WriteString(L" --> \"");
			fileOut.WriteString(str.c_str());
			fileOut.WriteString(L"\"");
		}
	}
}

void writeKey(CStdioFile &fileOut, HKEY hkRoot, const std::wstring &strPath, REGSAM sam)
{
	HKEY hkTestExist;

	if (ERROR_SUCCESS != RegOpenKey(hkRoot, strPath.c_str(), &hkTestExist))
	{
		fileOut.WriteString(getSamName(sam));
		fileOut.WriteString(L" ");
		fileOut.WriteString(getRootName(hkRoot));
		fileOut.WriteString(L"\\");
		fileOut.WriteString(strPath.c_str());
		fileOut.WriteString(L"\\ -- Does not exist\n");
		return;
	}

	RegCloseKey(hkTestExist);

	fileOut.WriteString(getSamName(sam));
	fileOut.WriteString(L" ");
	fileOut.WriteString(getRootName(hkRoot));
	fileOut.WriteString(L"\\");
	fileOut.WriteString(strPath.c_str());
	fileOut.WriteString(L"\\\n");

	{
		std::vector< std::wstring > vecValues;
		std::wstring strTemp;

		if (LeoHelpers::LeetRegEnumValue(hkRoot, strPath.c_str(), sam, &vecValues))
		{
			for (std::vector< std::wstring >::const_iterator nameIter = vecValues.begin(); nameIter != vecValues.end(); ++nameIter)
			{
				fileOut.WriteString(getSamName(sam));
				fileOut.WriteString(L" ");
				fileOut.WriteString(getRootName(hkRoot));
				fileOut.WriteString(L"\\");
				fileOut.WriteString(strPath.c_str());
				fileOut.WriteString(L": \"");
				fileOut.WriteString(nameIter->c_str());
				fileOut.WriteString(L"\"");

				DWORD dwType = REG_NONE;
				void *pvData = NULL;
				DWORD dataSize = 0;
				
				if (!LeoHelpers::LeetRegQueryValue(hkRoot, strPath.c_str(), nameIter->c_str(), sam, &dwType, &pvData, &dataSize))
				{
					DWORD dwErr = ::GetLastError();
					FormatToStdio(fileOut, L" could not be read (error %lu)\n", dwErr);
				}
				else
				{
					switch(dwType)
					{
					default:                             FormatToStdio(fileOut, L" has unknown type %lu\n", dwType); break;
					case REG_NONE:                       fileOut.WriteString(L" is REG_NONE\n"); break;
					case REG_BINARY:                     fileOut.WriteString(L" is REG_BINARY\n"); break;
					case REG_DWORD_BIG_ENDIAN:           fileOut.WriteString(L" is REG_DWORD_BIG_ENDIAN\n"); break;
					case REG_LINK:                       fileOut.WriteString(L" is REG_LINK\n"); break;
					case REG_RESOURCE_LIST:              fileOut.WriteString(L" is REG_RESOURCE_LIST\n"); break;
					case REG_FULL_RESOURCE_DESCRIPTOR:   fileOut.WriteString(L" is REG_FULL_RESOURCE_DESCRIPTOR\n"); break;
					case REG_RESOURCE_REQUIREMENTS_LIST: fileOut.WriteString(L" is REG_RESOURCE_REQUIREMENTS_LIST\n"); break;
					case REG_SZ:
						FormatToStdio(fileOut, L" is REG_SZ: \"%s\"", reinterpret_cast<wchar_t *>(pvData));
						writeExpLocStr(fileOut, reinterpret_cast<wchar_t *>(pvData));
						fileOut.WriteString(L"\n");
						break;
					case REG_EXPAND_SZ:
						if (! LeoHelpers::ExpandEnvVars(&strTemp, reinterpret_cast<wchar_t *>(pvData)) )
						{
							DWORD dwErr = ::GetLastError();
							FormatToStdio(fileOut, L" is REG_EXPAND_SZ: \"%s\" --> expansion failed (error %lu)\n", reinterpret_cast<wchar_t *>(pvData), dwErr);
						}
						else
						{
							FormatToStdio(fileOut, L" is REG_EXPAND_SZ: \"%s\" --> \"%s\"", reinterpret_cast<wchar_t *>(pvData), strTemp.c_str());
							writeExpLocStr(fileOut, strTemp.c_str());
							fileOut.WriteString(L"\n");
						}
						break;
					case REG_MULTI_SZ:
						if (reinterpret_cast<wchar_t *>(pvData)[0] == L'\0')
						{
							fileOut.WriteString(L" is REG_MULTI_SZ but empty\n");
						}
						else
						{
							fileOut.WriteString(L" is REG_MULTI_SZ:\n");
							int itemIdx = 0;
							for (wchar_t *szItem = reinterpret_cast<wchar_t *>(pvData); *szItem; szItem += (wcslen(szItem) + 1))
							{
								FormatToStdio(fileOut, L"\t[%d]\t\"%s\"\n", itemIdx, szItem);
								++itemIdx;
							}
						}
						break;
					case REG_DWORD:
						FormatToStdio(fileOut, L" is REG_DWORD: %lu (0x%lx)\n", *reinterpret_cast<DWORD *>(pvData), *reinterpret_cast<DWORD *>(pvData));
						break;
					case REG_QWORD:
						FormatToStdio(fileOut, L" is REG_QWORD: %I64u (0x%I64x)\n", *reinterpret_cast<unsigned __int64 *>(pvData), *reinterpret_cast<unsigned __int64 *>(pvData));
						break;
					}

					delete[] pvData;
				}
			}
		}
	}

	{
		std::vector< std::wstring > vecKeys;

		if (LeoHelpers::LeetRegEnumKey(hkRoot, strPath.c_str(), NULL, sam, &vecKeys))
		{
			std::wstring strChildPath;

			for (std::vector< std::wstring >::const_iterator nameIter = vecKeys.begin(); nameIter != vecKeys.end(); ++nameIter)
			{
				strChildPath = strPath;
				strChildPath += L"\\";
				strChildPath += *nameIter;

				writeKey(fileOut, hkRoot, strChildPath, sam);
			}
		}
	}
}

bool addToSet(SetNoCase &setDest, HKEY hkRoot, const std::wstring &strPath, const wchar_t *szValName, REGSAM sam, std::wstring *pstrOutValueInserted)
{
	std::wstring strValue;

	if (LeoHelpers::LeetRegQueryStringValue(hkRoot, strPath.c_str(), szValName, sam, &strValue))
	{
		if (pstrOutValueInserted)
		{
			*pstrOutValueInserted = strValue;
		}

		if (setDest.insert(strValue).second)
		{
			return true;
		}
	}
	else
	{
		if (pstrOutValueInserted)
		{
			pstrOutValueInserted->clear();
		}
	}

	return false;
}

void doTopLevel(const std::wstring &strExt, bool bIsFileExt, CStdioFile &fileOut, REGSAM *sams, size_t samCount)
{
	// There's some intentionally redundant stuff here, just in case it exposes something weird/unexpected
	// or something that changes in a later OS version. e.g. It if we've got values out of HKCU and HKLM
	// then the HKCR value should agree with at least one of them, but we get it anyway, just in case.
	// Same with the 64-bit / 32-bit stuff, where most (though not quite all) of these reg keys are shared
	// for 64-bit / 32-bit, at the time of writing. (Some of the 64/32 registry reflection stuff did change
	// in Windows 7, though only affecting COM really, so it could change again.)

	std::wstring strClassPath = L"Software\\Classes\\";
	strClassPath += strExt;

	std::wstring strExplorerFileExtsPath = L"Software\\Microsoft\\Windows\\CurrentVersion\\Explorer\\FileExts\\";
	strExplorerFileExtsPath += strExt;

	std::wstring strSystemFileAssociationsPath = L"Software\\Classes\\SystemFileAssociations\\";
	strSystemFileAssociationsPath += strExt;

	std::wstring strHkcrSystemFileAssociationsPath = L"SystemFileAssociations\\";
	strHkcrSystemFileAssociationsPath += strExt;

	SetNoCase setProgIDs;
	SetNoCase setPerceivedTypes;

	if (bIsFileExt)
	{
		for (size_t samIdx = 0; samIdx < samCount; ++samIdx)
		{
			REGSAM sam = sams[samIdx];

			addToSet(setProgIDs, HKEY_CURRENT_USER,  strClassPath, L"", sam, NULL);
			addToSet(setProgIDs, HKEY_LOCAL_MACHINE, strClassPath, L"", sam, NULL);
			addToSet(setProgIDs, HKEY_CLASSES_ROOT,  strExt,       L"", sam, NULL);

			std::wstring strExplorerFileExtsOWPI = strExplorerFileExtsPath;
			strExplorerFileExtsOWPI += L"\\OpenWithProgids";

			std::vector< std::wstring > vecValues;

			if (LeoHelpers::LeetRegEnumValue(HKEY_CURRENT_USER, strExplorerFileExtsOWPI.c_str(), sam, &vecValues))
			{
				for (std::vector< std::wstring >::const_iterator nameIter = vecValues.begin(); nameIter != vecValues.end(); ++nameIter)
				{
					setProgIDs.insert(*nameIter);
				}
			}

			if (LeoHelpers::LeetRegEnumValue(HKEY_LOCAL_MACHINE, strExplorerFileExtsOWPI.c_str(), sam, &vecValues))
			{
				for (std::vector< std::wstring >::const_iterator nameIter = vecValues.begin(); nameIter != vecValues.end(); ++nameIter)
				{
					setProgIDs.insert(*nameIter);
				}
			}

			std::wstring strExplorerFileExtsUserChoice = strExplorerFileExtsPath;
			strExplorerFileExtsUserChoice += L"\\UserChoice";

			addToSet(setProgIDs, HKEY_CURRENT_USER,  strExplorerFileExtsUserChoice, L"Progid", sam, NULL);
			addToSet(setProgIDs, HKEY_LOCAL_MACHINE, strExplorerFileExtsUserChoice, L"Progid", sam, NULL);
		}

		// Look up all CurVer values in all ProgIds and add the results to the list of ProgIDs
		// This lookup is recursive even though no version-specific ProgId should in turn link
		// to another version-specific ProgId via a CurVer value. The purpose of this app is
		// to gather as data to diagnose problems, not to get the minimal amount of data which
		// should actually be used.
		for (size_t samIdx = 0; samIdx < samCount; ++samIdx)
		{
			REGSAM sam = sams[samIdx];

			setProgIDs.erase(L"");

			SetNoCase setProgIDsToCheck = setProgIDs;

			std::wstring strProgIDToCheck;
			std::wstring strProgID;
			std::wstring strProgIDClassPath;

			while(!setProgIDsToCheck.empty())
			{
				strProgIDToCheck = *setProgIDsToCheck.begin();

				strProgIDClassPath = L"Software\\Classes\\";
				strProgIDClassPath += strProgIDToCheck;
				strProgIDClassPath += L"\\CurVer";

				if (addToSet(setProgIDs, HKEY_CURRENT_USER, strProgIDClassPath, L"", sam, &strProgID)
				&&	!strProgID.empty())
				{
					setProgIDsToCheck.insert(strProgID);
				}

				if (addToSet(setProgIDs, HKEY_LOCAL_MACHINE, strProgIDClassPath, L"", sam, &strProgID)
				&&	!strProgID.empty())
				{
					setProgIDsToCheck.insert(strProgID);
				}

				setProgIDsToCheck.erase(strProgIDToCheck);
			}
		}

		// Look up PerceviedType values in all ProgIDs
		for (size_t samIdx = 0; samIdx < samCount; ++samIdx)
		{
			REGSAM sam = sams[samIdx];

			addToSet(setPerceivedTypes, HKEY_CURRENT_USER,  strClassPath, L"PerceivedType", sam, NULL);
			addToSet(setPerceivedTypes, HKEY_LOCAL_MACHINE, strClassPath, L"PerceivedType", sam, NULL);
			addToSet(setPerceivedTypes, HKEY_CLASSES_ROOT,  strExt,       L"PerceivedType", sam, NULL);

			setProgIDs.erase(L"");

			for (SetNoCase::const_iterator progIDIter = setProgIDs.begin(); progIDIter != setProgIDs.end(); ++progIDIter)
			{
				std::wstring strProgIdClassPath = L"Software\\Classes\\";
				strProgIdClassPath += *progIDIter;

				addToSet(setPerceivedTypes, HKEY_CURRENT_USER,  strProgIdClassPath, L"PerceivedType", sam, NULL);
				addToSet(setPerceivedTypes, HKEY_LOCAL_MACHINE, strProgIdClassPath, L"PerceivedType", sam, NULL);
				addToSet(setPerceivedTypes, HKEY_CLASSES_ROOT,  *progIDIter,        L"PerceivedType", sam, NULL);
			}
		}
	}

	for (size_t samIdx = 0; samIdx < samCount; ++samIdx)
	{
		REGSAM sam = sams[samIdx];

		fileOut.WriteString(L"\n**********************************************************************\n\n");

		writeKey(fileOut, HKEY_CLASSES_ROOT,  strExt,       sam); fileOut.WriteString(L"----------------------------------------------------------------------\n");
		writeKey(fileOut, HKEY_CURRENT_USER,  strClassPath, sam); fileOut.WriteString(L"----------------------------------------------------------------------\n");
		writeKey(fileOut, HKEY_LOCAL_MACHINE, strClassPath, sam);

		if (bIsFileExt)
		{
			fileOut.WriteString(L"======================================================================\n");

			writeKey(fileOut, HKEY_CURRENT_USER,  strExplorerFileExtsPath, sam); fileOut.WriteString(L"----------------------------------------------------------------------\n");
			writeKey(fileOut, HKEY_LOCAL_MACHINE, strExplorerFileExtsPath, sam);

			fileOut.WriteString(L"======================================================================\n");

			writeKey(fileOut, HKEY_CLASSES_ROOT,  strHkcrSystemFileAssociationsPath, sam); fileOut.WriteString(L"----------------------------------------------------------------------\n");
			writeKey(fileOut, HKEY_CURRENT_USER,  strSystemFileAssociationsPath,     sam); fileOut.WriteString(L"----------------------------------------------------------------------\n");
			writeKey(fileOut, HKEY_LOCAL_MACHINE, strSystemFileAssociationsPath,     sam);

			setProgIDs.erase(L"");

			for (SetNoCase::const_iterator progIDIter = setProgIDs.begin(); progIDIter != setProgIDs.end(); ++progIDIter)
			{
				fileOut.WriteString(L"======================================================================\n");

				std::wstring strProgIDClassPath = L"Software\\Classes\\";
				strProgIDClassPath += *progIDIter;

				writeKey(fileOut, HKEY_CLASSES_ROOT,  *progIDIter,        sam); fileOut.WriteString(L"----------------------------------------------------------------------\n");
				writeKey(fileOut, HKEY_CURRENT_USER,  strProgIDClassPath, sam); fileOut.WriteString(L"----------------------------------------------------------------------\n");
				writeKey(fileOut, HKEY_LOCAL_MACHINE, strProgIDClassPath, sam);
			}

			setPerceivedTypes.erase(L"");

			for (SetNoCase::const_iterator perceivedTypeIter = setPerceivedTypes.begin(); perceivedTypeIter != setPerceivedTypes.end(); ++perceivedTypeIter)
			{
				fileOut.WriteString(L"======================================================================\n");

				std::wstring strPTviaCR = L"SystemFileAssociations\\";
				strPTviaCR += *perceivedTypeIter;

				std::wstring strPTviaSC = L"Software\\Classes\\";
				strPTviaSC += strPTviaCR;

				writeKey(fileOut, HKEY_CLASSES_ROOT,  strPTviaCR, sam); fileOut.WriteString(L"----------------------------------------------------------------------\n");
				writeKey(fileOut, HKEY_CURRENT_USER,  strPTviaSC, sam); fileOut.WriteString(L"----------------------------------------------------------------------\n");
				writeKey(fileOut, HKEY_LOCAL_MACHINE, strPTviaSC, sam);
			}
		}
	}
}

void CFileTypeDiagDlg::OnBnClickedOk()
{
	if (UpdateData(TRUE))
	{
		m_cstrExtension.Trim(L" ");

		bool bIsNoExtSpecial = (0 == m_cstrExtension.Compare(L"."));

		m_cstrExtension.Trim(L". ");

		bool bIsFileExt = m_cstrExtension.IsEmpty() ? false : true;

		CFileDialog fileDlg(FALSE, L"txt", bIsFileExt ? LPCTSTR(m_cstrExtension) : (bIsNoExtSpecial ? L"NoExtensions" : L"Folders"));

	//	if (m_cstrExtension.IsEmpty())
	//	{
	//		this->MessageBox(L"Please enter an extension.");
	//	}
	//	else
		if (fileDlg.DoModal() == IDOK)
		{
			CStdioFile fileOut;

			if (!fileOut.Open(fileDlg.GetPathName(), CFile::modeCreate | CFile::modeWrite | CFile::modeNoInherit | CFile::shareDenyWrite | CFile::typeText))
			{
				this->MessageBox(L"Could not write to specified file.");
			}
			else
			{
				fileOut.WriteString(L"\n**********************************************************************\n\n");
				fileOut.WriteString(L"FileTypeDiag version ");

				std::wstring strVersion;
				if (!LeoHelpers::GetVersionString(&strVersion, NULL, NULL))
				{
					strVersion = L"<couldn't get version>)";
				}

				fileOut.WriteString(strVersion.c_str());
				fileOut.WriteString(L"\nhttp://www.pretentiousname.com/miscsoft/index.html#FileTypeDiag\n");
				fileOut.WriteString(L"\n**********************************************************************\n\n");

				FormatToStdio(fileOut, L"      OS Bitness: %s\n", Is64BitWindows() ? L"64-bit" : L"32-bit");

				OSVERSIONINFOEX osvie = {0};
				osvie.dwOSVersionInfoSize = sizeof(osvie);

				if (::GetVersionEx(reinterpret_cast< OSVERSIONINFO * >(&osvie)))
				{
					FormatToStdio(fileOut, L"    MajorVersion: %lu\n", osvie.dwMajorVersion);
					FormatToStdio(fileOut, L"    MinorVersion: %lu\n", osvie.dwMinorVersion);
					FormatToStdio(fileOut, L"     BuildNumber: %lu\n", osvie.dwBuildNumber);
					FormatToStdio(fileOut, L"      PlatformId: %lu\n", osvie.dwPlatformId);
					if (osvie.szCSDVersion == NULL)
					{
						fileOut.WriteString(L"      CSDVersion: <none>\n");
					}
					else
					{
						FormatToStdio(fileOut, L"      CSDVersion: \"%s\"\n", osvie.szCSDVersion);
					}
					FormatToStdio(fileOut, L"ServicePackMajor: %hu\n", osvie.wServicePackMajor);
					FormatToStdio(fileOut, L"ServicePackMinor: %hu\n", osvie.wServicePackMinor);
					FormatToStdio(fileOut, L"       SuiteMask: 0x%hx\n", osvie.wSuiteMask);
					FormatToStdio(fileOut, L"     ProductType: %hu\n", static_cast<WORD>(osvie.wProductType)); // Called wProductType but really a BYTE.
					FormatToStdio(fileOut, L"      (Reserved): %hu\n", static_cast<WORD>(osvie.wReserved)); // Called wProductType but really a BYTE.
				}

				fileOut.WriteString(L"\n**********************************************************************\n\n");

				REGSAM sams[2] = {KEY_WOW64_64KEY, KEY_WOW64_32KEY};
				size_t samCount = _countof(sams);

				if (!Is64BitWindows())
				{
					sams[0] = 0;
					sams[1] = 0;
					samCount = 1;
				}

				if (bIsFileExt)
				{
					std::wstring strExt = L".";
					strExt += LPCTSTR(m_cstrExtension);

					fileOut.WriteString(L"File-type dump for \"");
					fileOut.WriteString(strExt.c_str());
					fileOut.WriteString(L"\"\n");

					doTopLevel(L"*",                    false, fileOut, sams, samCount);
					doTopLevel(L"AllFilesystemObjects", false, fileOut, sams, samCount);
					doTopLevel(strExt.c_str(),          true,  fileOut, sams, samCount);
				}
				else if (bIsNoExtSpecial)
				{
					fileOut.WriteString(L"File-type dump for files without extensions\n");

					doTopLevel(L"*",                    false, fileOut, sams, samCount);
					doTopLevel(L"AllFilesystemObjects", false, fileOut, sams, samCount);
					doTopLevel(L".",                    false, fileOut, sams, samCount);
				}
				else
				{
					fileOut.WriteString(L"File-type dump for folders\n");

					doTopLevel(L"*",                    false, fileOut, sams, samCount);
					doTopLevel(L"AllFilesystemObjects", false, fileOut, sams, samCount);
					doTopLevel(L"Folder",               false, fileOut, sams, samCount);
					doTopLevel(L"Directory",            false, fileOut, sams, samCount);
					doTopLevel(L"Drive",                false, fileOut, sams, samCount);
				}

				fileOut.WriteString(L"\n**********************************************************************\n\n");
				fileOut.WriteString(L"HKLM RegisteredApplications\n");

				const wchar_t *szRegAppsPath = L"SOFTWARE\\RegisteredApplications";

				for (size_t samIdx = 0; samIdx < samCount; ++samIdx)
				{
					REGSAM sam = sams[samIdx];

					fileOut.WriteString(L"\n**********************************************************************\n\n");

					writeKey(fileOut, HKEY_LOCAL_MACHINE, szRegAppsPath, sam);

					std::wstring strSingleApp;
					std::vector< std::wstring > vecRegApps;

					if (LeoHelpers::LeetRegEnumValue(HKEY_LOCAL_MACHINE, szRegAppsPath, sam, &vecRegApps))
					{
						for(std::vector< std::wstring >::const_iterator iterAppVal = vecRegApps.begin(); iterAppVal != vecRegApps.end(); ++iterAppVal)
						{
							fileOut.WriteString(L"----------------------------------------------------------------------\n");

							if (LeoHelpers::LeetRegQueryStringValue(HKEY_LOCAL_MACHINE, szRegAppsPath, iterAppVal->c_str(), sam, &strSingleApp))
							{
								writeKey(fileOut, HKEY_LOCAL_MACHINE, strSingleApp.c_str(), sam);
							}
						}
					}
				}

				fileOut.WriteString(L"\n**********************************************************************\n\n");

				fileOut.Close();

				m_cstrExtension = L"";
				UpdateData(FALSE);
			}
		}

//		OnOK();
	}
}
